@@ -44,27 +44,30 @@ class Location(ModelSQL, ModelView):
             'readonly': "not active",
             'required': "type == 'warehouse'",
         },
-        domain="[('type','=','storage'), ['OR', " \
-                "('parent', 'child_of', [active_id]), " \
-                "('parent', '=', False)]]")
+        domain=["('type','=','storage')",
+            ['OR',
+                "('parent', 'child_of', [active_id])",
+                "('parent', '=', False)"]])
     output_location = fields.Many2One(
         "stock.location", "Output", states={
             'invisible': "type != 'warehouse'",
             'readonly': "not active",
             'required': "type == 'warehouse'",
         },
-        domain="[('type','=','storage'), ['OR', " \
-                "('parent', 'child_of', [active_id]), " \
-                "('parent', '=', False)]]")
+        domain=["('type','=','storage')",
+            ['OR',
+                "('parent', 'child_of', [active_id])",
+                "('parent', '=', False)"]])
     storage_location = fields.Many2One(
         "stock.location", "Storage", states={
             'invisible': "type != 'warehouse'",
             'readonly': "not active",
             'required': "type == 'warehouse'",
         },
-        domain="[('type','=','storage'), ['OR', " \
-                "('parent', 'child_of', [active_id]), " \
-                "('parent', '=', False)]]")
+        domain=["('type','=','storage')",
+            ['OR',
+                "('parent', 'child_of', [active_id])",
+                "('parent', '=', False)"]])
     quantity = fields.Function('get_quantity', type='float', string='Quantity')
     forecast_quantity = fields.Function('get_quantity', type='float',
                                         string='Forecast Quantity')
@@ -80,6 +83,7 @@ class Location(ModelSQL, ModelView):
             'recursive_locations': 'You can not create recursive locations!',
             'invalid_type_for_moves': 'A location with existing moves ' \
                 'cannot be changed to a type that does not support moves.',
+            'child_of_warehouse': 'Location "%s" must be a child of warehouse "%s"!',
         })
 
     def init(self, cursor, module_name):
@@ -222,8 +226,91 @@ class Location(ModelSQL, ModelView):
             ids = [ids]
         locations = self.browse(cursor, user, ids, context=context)
         self._set_warehouse_parent(cursor, user, locations, context=context)
+
+        check_wh = self.search(
+            cursor, user,
+            [('type', '=', 'warehouse'),
+             ['OR',
+              ('storage_location', 'in', ids),
+              ('input_location', 'in', ids),
+              ('output_location', 'in', ids)
+              ]],
+            context=context)
+
+        warehouses = self.browse(cursor, user, check_wh, context=context)
+        fields = ('storage_location', 'input_location', 'output_location')
+        wh2childs = {}
+        for warehouse in warehouses:
+            in_out_sto = (warehouse[f].id for f in fields)
+            for location in locations:
+                if location.id not in in_out_sto:
+                    continue
+                childs = wh2childs.setdefault(
+                    warehouse.id,
+                    self.search(
+                        cursor, user, [('parent', 'child_of', warehouse.id)],
+                        context=context))
+                if location.id not in childs:
+                    self.raise_user_error(
+                        cursor, 'child_of_warehouse',
+                        (location.name, warehouse.name), context=context)
+
         return res
 
+    def copy(self, cursor, user, ids, default=None, context=None):
+        if default is None:
+            default = {}
+        int_id = False
+        if isinstance(ids, (int, long)):
+            int_id = True
+            ids = [ids]
+
+        default['left'] = 0
+        default['right'] = 0
+
+        res = []
+        locations = self.browse(cursor, user, ids, context=context)
+        for location in locations:
+            if location.type == 'warehouse':
+
+                wh_default = default.copy()
+                wh_default['type'] = 'view'
+                wh_default['input_location'] = False
+                wh_default['output_location'] = False
+                wh_default['storage_location'] = False
+                wh_default['childs'] = False
+
+                new_id = super(Location, self).copy(
+                    cursor, user, location.id, default=wh_default,
+                    context=context)
+
+                child_context = context and context.copy() or {}
+                child_context['cp_warehouse_locations'] = {
+                    'input_location': location.input_location.id,
+                    'output_location': location.output_location.id,
+                    'storage_location': location.storage_location.id}
+                child_context['cp_warehouse_id'] = new_id
+
+                self.copy(
+                    cursor, user, [c.id for c in location.childs],
+                    default={'parent':new_id}, context=child_context)
+                self.write(
+                    cursor, user, new_id, {'type': 'warehouse'}, context=context)
+            else:
+                new_id = super(Location, self).copy(
+                    cursor, user, location.id, default=default, context=context)
+                warehouse_locations = context.get('cp_warehouse_locations', {})
+                if location.id in warehouse_locations.values():
+                    for field, loc_id in warehouse_locations.iteritems():
+                        if loc_id == location.id:
+                            self.write(
+                                cursor, user, context['cp_warehouse_id'],
+                                {field: new_id}, context=context)
+
+            res.append(new_id)
+
+        return int_id and res[0] or res
+
 Location()
 
 
@@ -295,13 +382,13 @@ class OpenProduct(Wizard):
                 context=context)
         res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
 
-        if context == None: context = {}
-        context['locations'] = data['ids']
+        ctx = {}
+        ctx['locations'] = data['ids']
         if data['form']['forecast_date']:
-            context['stock_date_end'] = data['form']['forecast_date']
+            ctx['stock_date_end'] = data['form']['forecast_date']
         else:
-            context['stock_date_end'] = datetime.date.max
-        res['context'] = str(context)
+            ctx['stock_date_end'] = datetime.date.max
+        res['context'] = str(ctx)
 
         return res
 
