@@ -4,7 +4,6 @@
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.backend import TableHandler
 from decimal import Decimal
-import datetime
 
 STATES = {
     'readonly': "(state in ('cancel', 'done'))",
@@ -21,8 +20,8 @@ class Move(ModelSQL, ModelView):
                 'from_location', 'to_location'],
             domain=[('type', '!=', 'service')])
     uom = fields.Many2One("product.uom", "Uom", required=True, states=STATES,
-            domain="[('category', '=', " \
-                    "(product, 'product.default_uom.category'))]",
+            domain=["('category', '=', " \
+                    "(product, 'product.default_uom.category'))"],
             context="{'category': (product, 'product.default_uom.category')}",
             on_change=['product', 'currency', 'uom', 'company',
                 'from_location', 'to_location'])
@@ -32,23 +31,23 @@ class Move(ModelSQL, ModelView):
             digits="(16, unit_digits)", states=STATES)
     from_location = fields.Many2One("stock.location", "From Location", select=1,
             required=True, states=STATES,
-            domain="[('type', 'not in', " \
-                    "('warehouse', 'view'))]")
+            domain=["('type', 'not in', " \
+                    "('warehouse', 'view'))"])
     to_location = fields.Many2One("stock.location", "To Location", select=1,
             required=True, states=STATES,
-            domain="[('type', 'not in', " \
-                    "('warehouse', 'view'))]")
-    packing_in = fields.Many2One('stock.packing.in', 'Supplier Shipment',
+            domain=["('type', 'not in', " \
+                    "('warehouse', 'view'))"])
+    shipment_in = fields.Many2One('stock.shipment.in', 'Supplier Shipment',
             readonly=True, select=1, ondelete='CASCADE')
-    packing_out = fields.Many2One('stock.packing.out', 'Customer Shipment',
+    shipment_out = fields.Many2One('stock.shipment.out', 'Customer Shipment',
             readonly=True, select=1, ondelete='CASCADE')
-    packing_out_return = fields.Many2One('stock.packing.out.return',
+    shipment_out_return = fields.Many2One('stock.shipment.out.return',
             'Customer Return Shipment', readonly=True, select=1,
             ondelete='CASCADE')
-    packing_in_return = fields.Many2One('stock.packing.in.return',
+    shipment_in_return = fields.Many2One('stock.shipment.in.return',
             'Supplier Return Shipment', readonly=True, select=1,
             ondelete='CASCADE')
-    packing_internal = fields.Many2One('stock.packing.internal',
+    shipment_internal = fields.Many2One('stock.shipment.internal',
             'Internal Shipment', readonly=True, select=1, ondelete='CASCADE')
     planned_date = fields.Date("Planned Date", states=STATES, select=2)
     effective_date = fields.Date("Effective Date", readonly=True, select=2)
@@ -61,7 +60,8 @@ class Move(ModelSQL, ModelView):
     company = fields.Many2One('company.company', 'Company', required=True,
             states={
                 'readonly': "state != 'draft'",
-            }, domain="[('id', '=', context.get('company', 0))]")
+            }, domain=["('id', 'company' in context and '=' or '!=', " \
+                    "context.get('company', 0))"])
     unit_price = fields.Numeric('Unit Price', digits=(16, 4),
             states={
                 'invisible': "not unit_price_required",
@@ -87,16 +87,16 @@ class Move(ModelSQL, ModelView):
             ('check_from_to_locations',
                 'CHECK(from_location != to_location)',
                 'Source and destination location must be different'),
-            ('check_packing',
-                'CHECK((COALESCE(packing_in, 0) / COALESCE(packing_in, 1) ' \
-                        '+ COALESCE(packing_out, 0) / ' \
-                            'COALESCE(packing_out, 1) ' \
-                        '+ COALESCE(packing_internal, 0) / ' \
-                            'COALESCE(packing_internal, 1) ' \
-                        '+ COALESCE(packing_in_return, 0) / ' \
-                            'COALESCE(packing_in_return, 1) ' \
-                        '+ COALESCE(packing_out_return, 0) / ' \
-                            'COALESCE(packing_out_return, 1)) ' \
+            ('check_shipment',
+                'CHECK((COALESCE(shipment_in, 0) / COALESCE(shipment_in, 1) ' \
+                        '+ COALESCE(shipment_out, 0) / ' \
+                            'COALESCE(shipment_out, 1) ' \
+                        '+ COALESCE(shipment_internal, 0) / ' \
+                            'COALESCE(shipment_internal, 1) ' \
+                        '+ COALESCE(shipment_in_return, 0) / ' \
+                            'COALESCE(shipment_in_return, 1) ' \
+                        '+ COALESCE(shipment_out_return, 0) / ' \
+                            'COALESCE(shipment_out_return, 1)) ' \
                         '<= 1)',
                 'Move can be on only one Shipment'),
         ]
@@ -113,11 +113,20 @@ class Move(ModelSQL, ModelView):
             })
 
     def init(self, cursor, module_name):
+        # Migration from 1.2: packing renamed into shipment
+        table  = TableHandler(cursor, self, module_name)
+        table.drop_constraint('check_packing')
+        for suffix in ('in', 'out', 'in_return', 'out_return', 'internal'):
+            old_column = 'packing_%s' % suffix
+            new_column = 'shipment_%s' % suffix
+            if table.column_exist(old_column):
+                table.index_action(old_column, action='remove')
+            table.drop_fk(old_column)
+            table.column_rename(old_column, new_column)
         super(Move, self).init(cursor, module_name)
 
-        table  = TableHandler(cursor, self, module_name)
-
         # Migration from 1.0 check_packing_in_out has been removed
+        table  = TableHandler(cursor, self, module_name)
         table.drop_constraint('check_packing_in_out')
 
     def default_planned_date(self, cursor, user, context=None):
@@ -418,6 +427,7 @@ class Move(ModelSQL, ModelView):
         location_obj = self.pool.get('stock.location')
         currency_obj = self.pool.get('currency.currency')
         company_obj = self.pool.get('company.company')
+        date_obj = self.pool.get('ir.date')
 
         if context is None:
             context = {}
@@ -430,7 +440,7 @@ class Move(ModelSQL, ModelView):
         ctx = context and context.copy() or {}
         ctx['locations'] = location_obj.search(
             cursor, user, [('type', '=', 'storage')], context=context)
-        ctx['stock_date_end'] = datetime.date.today()
+        ctx['stock_date_end'] = date_obj.today(cursor, user, context=context)
         product = product_obj.browse(cursor, user, product_id, context=ctx)
         qty = uom_obj.compute_qty(
             cursor, user, uom, quantity, product.default_uom, context=context)
@@ -460,12 +470,14 @@ class Move(ModelSQL, ModelView):
     def create(self, cursor, user, vals, context=None):
         location_obj = self.pool.get('stock.location')
         product_obj = self.pool.get('product.product')
+        date_obj = self.pool.get('ir.date')
 
         vals = vals.copy()
 
         if vals.get('state') == 'done':
             if not vals.get('effective_date'):
-                vals['effective_date'] = datetime.date.today()
+                vals['effective_date'] = date_obj.today(cursor, user,
+                        context=context)
             from_location = location_obj.browse(cursor, user,
                     vals['from_location'], context=context)
             to_location = location_obj.browse(cursor, user,
@@ -492,10 +504,13 @@ class Move(ModelSQL, ModelView):
 
         elif vals.get('state') == 'assigned':
             if not vals.get('effective_date'):
-                vals['effective_date'] = datetime.date.today()
+                vals['effective_date'] = date_obj.today(cursor, user,
+                        context=context)
         return super(Move, self).create(cursor, user, vals, context=context)
 
     def write(self, cursor, user, ids, vals, context=None):
+        date_obj = self.pool.get('ir.date')
+
         if context is None:
             context = {}
 
@@ -529,12 +544,14 @@ class Move(ModelSQL, ModelView):
                     if move.state in ('cancel', 'done'):
                         self.raise_user_error(cursor, 'set_state_assigned',
                                 context=context)
-                    vals['effective_date'] = datetime.date.today()
+                    vals['effective_date'] = date_obj.today(cursor, user,
+                            context=context)
                 elif vals['state'] == 'done':
                     if move.state in ('cancel'):
                         self.raise_user_error(cursor, 'set_state_done',
                                 context=context)
-                    vals['effective_date'] = datetime.date.today()
+                    vals['effective_date'] = date_obj.today(cursor, user,
+                            context=context)
 
                     if move.from_location.type == 'supplier' \
                             and move.state != 'done' \
@@ -618,7 +635,8 @@ class Move(ModelSQL, ModelView):
         if context is None:
             context = {}
 
-        cursor.execute('LOCK TABLE stock_move')
+        if cursor.has_lock():
+            cursor.execute('LOCK TABLE stock_move')
 
         local_ctx = context and context.copy() or {}
         local_ctx['stock_date_end'] = date_obj.today(cursor, user,
