@@ -23,6 +23,38 @@ class Template(ModelSQL, ModelView):
                 res[template.id] += product[name]
         return res
 
+    def __init__(self):
+        super(Template, self).__init__()
+        self._error_messages.update({
+                'change_default_uom': 'You cannot change the default uom for '\
+                    'a product which is associated to stock moves.',
+            })
+
+    def write(self, cursor, user, ids, vals, context=None):
+        move_obj = self.pool.get('stock.move')
+        if not  vals.get("default_uom"):
+            return super(Template, self).write(cursor, user, ids, vals,
+                    context=context)
+
+        for i in range(0, len(ids), cursor.IN_MAX):
+            sub_ids = ids[i:i + cursor.IN_MAX]
+            res = self.search(cursor, user, [
+                    ('id', 'in', sub_ids),
+                    ('default_uom', '!=', vals['default_uom']),
+                    ], context=context)
+
+            if res:
+                res = move_obj.search(cursor, user, [
+                        ('product.template', 'in', res),
+                        ], limit=1, context=context)
+                if res:
+                    self.raise_user_error(cursor, 'change_default_uom',
+                            context=context)
+
+        return super(Template, self).write(cursor, user, ids, vals,
+                context=context)
+
+
 Template()
 
 class Product(ModelSQL, ModelView):
@@ -40,6 +72,7 @@ class Product(ModelSQL, ModelView):
             return dict([(id, 0.0) for id in ids])
 
         if name == 'quantity' and \
+                context.get('stock_date_end') and \
                 context.get('stock_date_end') > \
                 date_obj.today(cursor, user, context=context):
 
@@ -80,15 +113,21 @@ class Product(ModelSQL, ModelView):
 
         if not (context and context.get('locations') and domain):
             return []
-        if (name != 'forecast_quantity') and context.get('stock_date_end'):
-            if context['stock_date_end'] != date_obj.today(cursor, user,
-                    context=context):
-                context = context.copy()
-                del context['stock_date_end']
 
-        if name == 'forecast_quantity' and not context.get('stock_date_end'):
+        if name == 'quantity' and \
+                context.get('stock_date_end') and \
+                context.get('stock_date_end') > \
+                date_obj.today(cursor, user, context=context):
+
             context = context.copy()
-            context['stock_date_end'] = datetime.date.max
+            context['stock_date_end'] = date_obj.today(
+                cursor, user, context=context)
+
+        if name == 'forecast_quantity':
+            context = context.copy()
+            context['forecast'] = True
+            if not context.get('stock_date_end'):
+                context['stock_date_end'] = datetime.date.max
 
         pbl = self.products_by_location(
             cursor, user, location_ids=context['locations'], with_childs=True,
@@ -143,7 +182,7 @@ class Product(ModelSQL, ModelView):
         today = date_obj.today(cursor, user, context=context)
 
         if not location_ids:
-            return []
+            return {}
         if context is None:
             context= {}
         # Skip warehouse location in favor of their storage location
@@ -193,10 +232,6 @@ class Product(ModelSQL, ModelView):
                     context['stock_date_end'],
                     context['stock_date_end'],
                     ]
-        # infinite date end: take all states for the moves
-        elif context['stock_date_end'] == datetime.date.max:
-            state_date_clause = 'state in (%s, %s, %s)'
-            state_date_vals = ['done', 'assigned', 'draft']
         # future date end: filter move on state done and date
         # before today, or on all state and date between today and
         # date_end.
@@ -239,7 +274,8 @@ class Product(ModelSQL, ModelView):
                 ')'
 
             state_date_vals = [
-                'done', 'assigned', today, today,
+                'done', context.get('stock_assign') and 'assigned' or 'done',
+                today, today,
                 'done', 'assigned', 'draft',
                 context['stock_date_end'], today,
                 context['stock_date_end'], today,
@@ -331,25 +367,36 @@ class Product(ModelSQL, ModelView):
             where_vals = args
         else:
             where_clause = " IN (" + \
-                ",".join(["%s" for i in location_ids]) + ") "
+                ",".join(('%s',) * len(location_ids)) + ") "
             where_vals = location_ids[:]
 
         if move_query:
             where_clause += " AND " + move_query + " "
             where_vals += move_val
 
+        product_template_join = ""
         if product_ids:
             where_clause += "AND product in (" + \
-                ",".join(["%s" for i in product_ids]) + ")"
+                ",".join(('%s',) * len(product_ids)) + ")"
             where_vals += product_ids
+        else:
+            where_clause += "AND product_template.active = %s"
+            where_vals.append(True)
+            product_template_join = \
+                    "JOIN product_product "\
+                        "ON (stock_move.product = product_product.id) "\
+                    "JOIN product_template "\
+                        "ON (product_product.template = "\
+                            "product_template.id) "\
+
 
         if context.get('stock_destinations'):
             destinations = context.get('stock_destinations')
             dest_clause_from = " AND from_location in ("
-            dest_clause_from += ",".join("%s" for i in destinations)
+            dest_clause_from += ",".join(('%s',) * len(destinations))
             dest_clause_from += ") "
             dest_clause_to = " AND to_location in ("
-            dest_clause_to += ",".join("%s" for i in destinations)
+            dest_clause_to += ",".join(('%s',) * len(destinations))
             dest_clause_to += ") "
             dest_vals = destinations
 
@@ -362,14 +409,14 @@ class Product(ModelSQL, ModelView):
                 "FROM ( "\
                     "SELECT to_location AS location, product, uom, "\
                         "sum(quantity) AS quantity "\
-                    "FROM stock_move "\
+                    "FROM stock_move " + product_template_join + \
                     "WHERE (%s) " \
                         "AND to_location %s "\
                     "GROUP BY to_location, product ,uom "\
                     "UNION  "\
                     "SELECT from_location AS location, product, uom, "\
                         "-sum(quantity) AS quantity "\
-                    "FROM stock_move "\
+                    "FROM stock_move " + product_template_join + \
                     "WHERE (%s) " \
                         "AND from_location %s "\
                     "GROUP BY from_location, product, uom "\
@@ -392,13 +439,10 @@ class Product(ModelSQL, ModelView):
                 if line[position] not in id_list:
                     id_list.append(line[position])
 
-        if not product_ids:
-            product_ids = self.pool.get("product.product").search(
-                cursor, user, [], context=context)
         uom_by_id = dict([(x.id, x) for x in uom_obj.browse(
                 cursor, user, uom_ids, context=context)])
         default_uom = dict((x.id, x.default_uom) for x in product_obj.browse(
-                cursor, user, product_ids, context=context))
+                cursor, user, product_ids or res_product_ids, context=context))
 
         for line in raw_lines:
             location, product, uom, quantity = line
@@ -451,7 +495,13 @@ class Product(ModelSQL, ModelView):
 
         # Complete result with missing products if asked
         if not skip_zero:
-            keys = ((l,p) for l in location_ids for p in product_ids)
+            # Search for all products, even if not linked with moves
+            if product_ids:
+                all_product_ids = product_ids
+            else:
+                all_product_ids = self.pool.get("product.product").search(
+                    cursor, user, [], context=context)
+            keys = ((l,p) for l in location_ids for p in all_product_ids)
             for location_id, product_id in keys:
                 if (location_id, product_id) not in res:
                     res[(location_id, product_id)] = 0.0
@@ -473,7 +523,7 @@ class Product(ModelSQL, ModelView):
         location_obj = self.pool.get('stock.location')
         locations = location_obj.browse(cursor, user, context.get('locations'),
                                         context=context)
-        return value + " (" + ",".join([l.name for l in locations]) + ")"
+        return value + " (" + ",".join(l.name for l in locations) + ")"
 
 Product()
 
@@ -529,13 +579,13 @@ class OpenLocation(Wizard):
                 context=context)
         res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
 
-        if context == None: context = {}
-        context['product'] = data['id']
+        ctx = {}
+        ctx['product'] = data['id']
         if data['form']['forecast_date']:
-            context['stock_date_end'] = data['form']['forecast_date']
+            ctx['stock_date_end'] = data['form']['forecast_date']
         else:
-            context['stock_date_end'] = datetime.date.max
-        res['context'] = str(context)
+            ctx['stock_date_end'] = datetime.date.max
+        res['context'] = str(ctx)
 
         return res
 
